import json
import logging

from django.http import HttpResponse
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View

from ..models import AdyenNotification

logger = logging.getLogger(__name__)


class NotificationView(View):
    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        return super(NotificationView, self).dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        logger.debug(_('New notification'))
        logger.debug(request.POST)
        json_params = json.dumps(request.POST)
        if request.POST:
            notification = AdyenNotification.objects.create(notification=json_params)
            logger.debug(_('Notification saved | id: %s'), notification.id)
        return HttpResponse('[accepted]')



from django import forms

from djadyen import settings


class PaymentForm(forms.Form):
    skinCode = forms.CharField(initial=settings.ADYEN_SKIN_CODE)
    currencyCode = forms.CharField(initial=settings.ADYEN_CURRENCYCODE)
    merchantAccount = forms.CharField(initial=settings.ADYEN_MERCHANT_ACCOUNT)

    sessionValidity = forms.CharField()
    merchantReference = forms.CharField()
    paymentAmount = forms.CharField()

    merchantSig = forms.CharField()

    def __init__(self, *args, **kwargs):
        super(PaymentForm, self).__init__(*args, **kwargs)

        initial = kwargs.get('initial')

        if initial.get('shopperEmail'):
            self.fields['shopperEmail'] = forms.CharField()
        if initial.get('shipBeforeDate'):
            self.fields['shipBeforeDate'] = forms.CharField()
        if initial.get('merchantReturnData'):
            self.fields['merchantReturnData'] = forms.CharField()
        if initial.get('resURL'):
            self.fields['resURL'] = forms.CharField()
        if initial.get('brandCode'):
            self.fields['brandCode'] = forms.CharField()
        if initial.get('issuerId'):
            self.fields['issuerId'] = forms.CharField()



import logging
from uuid import uuid4

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from djadyen import settings
from .choices import Status


logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class AdyenNotification(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    notification = models.TextField()
    is_processed = models.BooleanField(default=False)
    processed_at = models.DateTimeField(blank=True, null=True)

    def __str__(self):
        return self.created_at.strftime('%c')


@python_2_unicode_compatible
class AdyenPaymentOption(models.Model):
    name = models.CharField(max_length=200, default="")
    adyen_name = models.CharField(max_length=200, default="")
    guid = models.CharField(max_length=36, verbose_name=_('GUID'), default="")
    image = models.ImageField(null=True)
    is_active = models.BooleanField(default=False)

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class AdyenIssuer(models.Model):
    payment_option = models.ForeignKey(AdyenPaymentOption)
    name = models.CharField(max_length=200, default="")
    adyen_id = models.CharField(max_length=200, default="")

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class AdyenOrder(models.Model):
    status = models.CharField(max_length=200, choices=Status.choices, default=Status.Created)
    created_on = models.DateTimeField(auto_now_add=True)
    reference = models.CharField(max_length=200, default="", blank=True)
    psp_reference = models.CharField(max_length=200, default="", blank=True)
    email = models.EmailField()

    payment_option = models.ForeignKey(AdyenPaymentOption, blank=True, null=True)
    issuer = models.ForeignKey(AdyenIssuer, null=True, blank=True)

    __old_status = None

    class Meta:
        abstract = True

    def __str__(self):
        return '{}'.format(self.reference)

    def __init__(self, *args, **kwargs):
        super(AdyenOrder, self).__init__(*args, **kwargs)
        self.__old_status = self.status

    def save(self, can_change=False, *args, **kwargs):
        if not self.reference:
            self.reference = uuid4()

        if settings.ADYEN_REFETCH_OLD_STATUS and self.id:
            self.__old_status = self._meta.model.objects.get(pk=self.id).status

        if self.status != self.__old_status:
            if self.__old_status == Status.Authorised:
                logger.warning(
                    _("Order ref: %s | Tried to change the status from 'Authorised' to '%s'. "),
                    self.reference, self.status)
                self.status = self.__old_status
            else:
                logger.warning(
                    _("Order ref: %s | Changed the status from '%s' to '%s'"),
                    self.reference, self.__old_status, self.status)
                self.__old_status = self.status

        return super(AdyenOrder, self).save(*args, **kwargs)



from django.contrib import admin

from .models import AdyenIssuer, AdyenNotification, AdyenPaymentOption


@admin.register(AdyenNotification)
class AdyenNotificationAdmin(admin.ModelAdmin):
    list_display = ('created_at', 'is_processed', 'processed_at')
    list_filter = ('is_processed', 'created_at', 'processed_at')


@admin.register(AdyenPaymentOption)
class PaymentOptionAdmin(admin.ModelAdmin):
    list_display = ('name', 'adyen_name', 'is_active')
    list_filter = ('is_active', )


@admin.register(AdyenIssuer)
class IssuerAdmin(admin.ModelAdmin):
    list_display = ('name', 'payment_option')
    list_filter = ('payment_option', )



from django.core.management import call_command
from django.test.testcases import TestCase

from djadyen.models import AdyenIssuer, AdyenPaymentOption


class SyncPaymentMethods(TestCase):
    def test_on_empty_database(self):
        self.assertEqual(AdyenPaymentOption.objects.count(), 0)
        self.assertEqual(AdyenIssuer.objects.count(), 0)

        with self.assertRaises(ValueError):
            call_command('sync_payment_methods')

        # self.assertEqual(AdyenPaymentOption.objects.count(), 3)
        # self.assertEqual(AdyenIssuer.objects.count(), 13)

    def test_on_existing_database(self):
        self.assertEqual(AdyenPaymentOption.objects.count(), 0)
        self.assertEqual(AdyenIssuer.objects.count(), 0)

        with self.assertRaises(ValueError):
            call_command('sync_payment_methods')

        # self.assertEqual(AdyenPaymentOption.objects.count(), 3)
        # self.assertEqual(AdyenIssuer.objects.count(), 13)

        # call_command('sync_payment_methods')

        # self.assertEqual(AdyenPaymentOption.objects.count(), 3)
        # self.assertEqual(AdyenIssuer.objects.count(), 13)



from django.conf.urls import url

from .views import ConfirmationView, Confirmation2View, Confirmation3View, MyAdyenRequestView, My2AdyenRequestView


urlpatterns = [
    url(r'^confirm/$', ConfirmationView.as_view(), name="confirm"),
    url(r'^confirm2/$', Confirmation2View.as_view(), name="confirm2"),
    url(r'^confirm3/$', Confirmation3View.as_view(), name="confirm3"),
    url(r'^(?P<reference>[\w-]+)/$', MyAdyenRequestView.as_view(), name="redirect"),
    url(r'^2/(?P<reference>[\w-]+)/$', My2AdyenRequestView.as_view(), name="redirect2"),
]



import factory

from djadyen.models import AdyenIssuer, AdyenNotification, AdyenPaymentOption
from ..app.models import Order


class NotificationFactory(factory.DjangoModelFactory):
    class Meta:
        model = AdyenNotification


class PaymentOptionsFactory(factory.DjangoModelFactory):
    name = factory.Faker('name')
    adyen_name = 'ideal'
    guid = 'ee112bdd-b03b-422f-8af2-085c64dd8698'

    class Meta:
        model = AdyenPaymentOption


class IssuerFactory(factory.DjangoModelFactory):
    payment_option = factory.SubFactory(PaymentOptionsFactory)
    name = factory.Faker('name')
    adyen_id = 1155

    class Meta:
        model = AdyenIssuer


class OrderFactory(factory.DjangoModelFactory):
    email = factory.Faker('email')

    class Meta:
        model = Order



import os

from setuptools import find_packages, setup

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='djadyen',
    version='0.1.5',
    license='BSD',

    # packaging
    install_requires=[
        'Django>=1.8',
        'django-choices',
        'requests',
    ],
    include_package_data=True,
    packages=find_packages(exclude=["tests"]),

    # tests
    test_suite='runtests.runtests',
    tests_require=['coverage'],

    # metadata
    description='A Django package to intergrade Adyen in your project.',
    url='https://github.com/maykinmedia/djadyen',
    author='Maykin Media, Jorik Kraaikamp',
    author_email='jorik.kraaikamp@maykinmedia.nl',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



from django.core.urlresolvers import reverse
from django.views.generic import TemplateView
from django.utils import timezone

from djadyen.choices import Status
from djadyen.views import AdyenRedirectView, AdyenResponseMixin

from .models import Order


class MyAdyenRequestView(AdyenRedirectView):
    model = Order

    def get_form_kwargs(self):
        order = self.get_object()
        params = self.get_signed_order_params(order)

        kwargs = super(MyAdyenRequestView, self).get_form_kwargs()
        kwargs.update({'initial': params})
        return kwargs

    def get_next_url(self): # This is to populate the resURL
        return reverse('confirm')


class My2AdyenRequestView(AdyenRedirectView):
    model = Order

    def get_form_kwargs(self):
        order = self.get_object()
        params = self.get_default_params(shipBeforeDate=timezone.now(), merchantReturnData='returnData')

        kwargs = super(My2AdyenRequestView, self).get_form_kwargs()
        kwargs.update({'initial': params})
        return kwargs


class ConfirmationView(AdyenResponseMixin, TemplateView):
    template_name = 'app/confirmation.html'
    model = Order

    def handle_authorised(self):
        self.order.status = Status.Authorised
        return self.done()

    def handle_pending(self):
        self.order.status = Status.Pending
        return self.done()

    def handle_refused(self):
        self.order.status = Status.Refused
        return self.done()

    def handle_error(self):
        self.order.status = Status.Error
        return self.done()

    def handle_canceled(self):
        self.order.status = Status.Cancel
        return self.done()

    def handle_default(self):
        if self.psp_reference:
            self.order.psp_reference = self.psp_reference


class Confirmation2View(AdyenResponseMixin, TemplateView):
    template_name = 'app/confirmation.html'
    auto_fetch = False


class Confirmation3View(AdyenResponseMixin, TemplateView):
    template_name = 'app/confirmation.html'
    auto_fetch = False

    def handle_authorised(self):
        return self.done()

    def handle_pending(self):
        return self.done()

    def handle_refused(self):
        return self.done()

    def handle_error(self):
        return self.done()

    def handle_canceled(self):
        return self.done()



from django.conf import settings


ADYEN_HOST_URL = getattr(settings, 'ADYEN_HOST_URL')
ADYEN_MERCHANT_ACCOUNT = getattr(settings, 'ADYEN_MERCHANT_ACCOUNT')
ADYEN_MERCHANT_SECRET = getattr(settings, 'ADYEN_MERCHANT_SECRET')
ADYEN_SKIN_CODE = getattr(settings, 'ADYEN_SKIN_CODE')

ADYEN_CURRENCYCODE = getattr(settings, 'ADYEN_CURRENCYCODE', 'EUR')
ADYEN_ENABLED = getattr(settings, 'ADYEN_ENABLED', True)
ADYEN_NEXT_STATUS = getattr(settings, 'ADYEN_NEXT_STATUS', 'AUTHORISED')
ADYEN_REFETCH_OLD_STATUS = getattr(settings, 'ADYEN_REFETCH_OLD_STATUS', False)
ADYEN_SESSION_MINUTES = getattr(settings, 'ADYEN_SESSION_MINUTES', 10)
ADYEN_URL = getattr(settings, 'ADYEN_URL', 'https://test.adyen.com/')
